@@ -327,7 +327,8 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
         final String action = WebConsoleUtil.getParameter( req, "action" );
         if ( "refreshPackages".equals( action ) )
         {
-            getPackageAdmin().refreshPackages( null );
+            // refresh packages and give it most 15 seconds to finish
+            BaseUpdateInstallHelper.refreshPackages( getPackageAdmin(), getBundleContext(), 15000L, null );
             success = true;
         }
         else if ( "install".equals( action ) )
@@ -384,8 +385,8 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                 }
                 else if ( "refresh".equals( action ) )
                 {
-                    // refresh bundle wiring
-                    refresh( bundle );
+                    // refresh bundle wiring and give at most 5 seconds to finish
+                    BaseUpdateInstallHelper.refreshPackages( getPackageAdmin(), getBundleContext(), 5000L, bundle );
                 }
                 else if ( "update".equals( action ) )
                 {
@@ -406,14 +407,6 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                     }
                 }
 
-                // let's wait a little bit to give the framework time
-                // to process our request
-                try {
-                    Thread.sleep(800);
-                } catch (InterruptedException e) {
-                    // we ignore this
-                }
-
                 // write the state only
                 resp.setContentType( "application/json" ); //$NON-NLS-1$
                 resp.setCharacterEncoding( "UTF-8" ); //$NON-NLS-1$
@@ -433,13 +426,6 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
 
         if ( success && null != getBundleContext() )
         {
-            // let's wait a little bit to give the framework time
-            // to process our request
-            try {
-                Thread.sleep(800);
-            } catch (InterruptedException e) {
-                // we ignore this
-            }
             final String pluginRoot = ( String ) req.getAttribute( WebConsoleConstants.ATTR_PLUGIN_ROOT );
             final String servicesRoot = getServicesRoot( req );
             try
@@ -456,7 +442,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
             super.doPost( req, resp );
         }
     }
-    
+
     private String getServicesRoot(HttpServletRequest request)
     {
         return ( ( String ) request.getAttribute( WebConsoleConstants.ATTR_APP_ROOT ) ) +
@@ -474,7 +460,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
             final long bundleId = Long.parseLong( pathInfo );
             if ( bundleId >= 0 )
             {
-                return getBundleContext().getBundle( bundleId );
+                return BundleContextUtil.getWorkingBundleContext(this.getBundleContext()).getBundle( bundleId );
             }
         }
         catch ( NumberFormatException nfe )
@@ -492,7 +478,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
             }
 
             // search
-            final Bundle[] bundles = getBundleContext().getBundles();
+            final Bundle[] bundles = BundleContextUtil.getWorkingBundleContext(this.getBundleContext()).getBundles();
             for(int i=0; i<bundles.length; i++)
             {
                 final Bundle bundle = bundles[i];
@@ -751,7 +737,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
 
     private final Bundle[] getBundles()
     {
-        return getBundleContext().getBundles();
+        return BundleContextUtil.getWorkingBundleContext(this.getBundleContext()).getBundles();
     }
 
 
@@ -800,24 +786,24 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
 
         jw.key( "props" );
         jw.array();
-        WebConsoleUtil.keyVal( jw, "Symbolic Name", bundle.getSymbolicName() );
-        WebConsoleUtil.keyVal( jw, "Version", headers.get( Constants.BUNDLE_VERSION ) );
-        WebConsoleUtil.keyVal( jw, "Bundle Location", bundle.getLocation() );
-        WebConsoleUtil.keyVal( jw, "Last Modification", new Date( bundle.getLastModified() ) );
+        Util.keyVal( jw, "Symbolic Name", bundle.getSymbolicName() );
+        Util.keyVal( jw, "Version", headers.get( Constants.BUNDLE_VERSION ) );
+        Util.keyVal( jw, "Bundle Location", bundle.getLocation() );
+        Util.keyVal( jw, "Last Modification", new Date( bundle.getLastModified() ) );
 
         String docUrl = ( String ) headers.get( Constants.BUNDLE_DOCURL );
         if ( docUrl != null )
         {
-            WebConsoleUtil.keyVal( jw, "Bundle Documentation", docUrl );
+            Util.keyVal( jw, "Bundle Documentation", docUrl );
         }
 
-        WebConsoleUtil.keyVal( jw, "Vendor", headers.get( Constants.BUNDLE_VENDOR ) );
-        WebConsoleUtil.keyVal( jw, "Copyright", headers.get( Constants.BUNDLE_COPYRIGHT ) );
-        WebConsoleUtil.keyVal( jw, "Description", headers.get( Constants.BUNDLE_DESCRIPTION ) );
+        Util.keyVal( jw, "Vendor", headers.get( Constants.BUNDLE_VENDOR ) );
+        Util.keyVal( jw, "Copyright", headers.get( Constants.BUNDLE_COPYRIGHT ) );
+        Util.keyVal( jw, "Description", headers.get( Constants.BUNDLE_DESCRIPTION ) );
 
-        WebConsoleUtil.keyVal( jw, "Start Level", getStartLevel( bundle ) );
+        Util.keyVal( jw, "Start Level", getStartLevel( bundle ) );
 
-        WebConsoleUtil.keyVal( jw, "Bundle Classpath", headers.get( Constants.BUNDLE_CLASSPATH ) );
+        Util.keyVal( jw, "Bundle Classpath", headers.get( Constants.BUNDLE_CLASSPATH ) );
 
         listFragmentInfo( jw, bundle, pluginRoot );
 
@@ -948,11 +934,11 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                     }
                 }
             }
-            WebConsoleUtil.keyVal( jw, "Exported Packages", val );
+            Util.keyVal( jw, "Exported Packages", val );
         }
         else
         {
-            WebConsoleUtil.keyVal( jw, "Exported Packages", "---" );
+            Util.keyVal( jw, "Exported Packages", "---" );
         }
 
         exports = packageAdmin.getExportedPackages( ( Bundle ) null );
@@ -1006,7 +992,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                 val.put( "None" );
             }
 
-            WebConsoleUtil.keyVal( jw, "Imported Packages", val );
+            Util.keyVal( jw, "Imported Packages", val );
         }
 
         if ( !usingBundles.isEmpty() )
@@ -1017,7 +1003,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                 Bundle usingBundle = ( Bundle ) ui.next();
                 val.put( getBundleDescriptor( usingBundle, pluginRoot ) );
             }
-            WebConsoleUtil.keyVal( jw, "Importing Bundles", val );
+            Util.keyVal( jw, "Importing Bundles", val );
         }
     }
 
@@ -1053,11 +1039,11 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                     Clause export = new Clause( pkgs[i].getName(), pkgs[i].getDirectives(), pkgs[i].getAttributes() );
                     collectExport( val, export.getName(), export.getAttribute( Constants.VERSION_ATTRIBUTE ) );
                 }
-                WebConsoleUtil.keyVal( jw, "Exported Packages", val );
+                Util.keyVal( jw, "Exported Packages", val );
             }
             else
             {
-                WebConsoleUtil.keyVal( jw, "Exported Packages", "---" );
+                Util.keyVal( jw, "Exported Packages", "---" );
             }
         }
 
@@ -1127,7 +1113,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                     val.put( "---" );
                 }
 
-                WebConsoleUtil.keyVal( jw, "Imported Packages", val );
+                Util.keyVal( jw, "Imported Packages", val );
             }
         }
     }
@@ -1175,7 +1161,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
             appendProperty( val, refs[i], Constants.SERVICE_DESCRIPTION, "Description" );
             appendProperty( val, refs[i], Constants.SERVICE_VENDOR, "Vendor" );
 
-            WebConsoleUtil.keyVal( jw, key, val);
+            Util.keyVal( jw, key, val);
         }
     }
 
@@ -1196,7 +1182,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
             val.put( header + ": " + value );
         }
 
-        WebConsoleUtil.keyVal( jw, "Manifest Headers", val );
+        Util.keyVal( jw, "Manifest Headers", val );
     }
 
     private static final String enableLineWrapping(final String value)
@@ -1231,7 +1217,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                 {
                     val.put( getBundleDescriptor( hostBundles[i], pluginRoot ) );
                 }
-                WebConsoleUtil.keyVal( jw, "Host Bundles", val );
+                Util.keyVal( jw, "Host Bundles", val );
             }
         }
         else
@@ -1244,7 +1230,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
                 {
                     val.put( getBundleDescriptor( fragmentBundles[i], pluginRoot ) );
                 }
-                WebConsoleUtil.keyVal( jw, "Fragments Attached", val );
+                Util.keyVal( jw, "Fragments Attached", val );
             }
         }
 
@@ -1456,14 +1442,6 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
         return val.toString();
     }
 
-
-    private void refresh( final Bundle bundle )
-    {
-        getPackageAdmin().refreshPackages( new Bundle[]
-            { bundle } );
-    }
-
-
     private void update( final Bundle bundle )
     {
         UpdateHelper t = new UpdateHelper( this, bundle, false );
@@ -1672,7 +1650,7 @@ public class BundlesServlet extends SimpleWebConsolePlugin implements OsgiManage
             }
             else
             {
-                Bundle[] bundles = getBundleContext().getBundles();
+                Bundle[] bundles = BundleContextUtil.getWorkingBundleContext(this.getBundleContext()).getBundles();
                 for ( int i = 0; i < bundles.length; i++ )
                 {
                     if ( ( bundles[i].getLocation() != null && bundles[i].getLocation().equals( location ) )
